/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedSemantikAdapter;

public class FormSolutionsEngineBasedAdapter implements EngineBasedSemantikAdapter {

	public static final String IDENTIFIER_KEY = "identifier";

	public static final String ASSISTANT = "assistant";
	public static final String ANLIEGEN_ID = "anliegenId";
	public static final String KOMMUNALVERWALTUNG_ID = "kommunalverwaltungId";

	@Autowired
	private List<FormSolutionsEngineBasedMapper> mappers;

	@Override
	public FormData parseFormData(FormData formData) {
		var processedFormData = formData;

		for (var mapper : mappers) {
			processedFormData = mapper.parseFormData(processedFormData);
		}

		return removeProcessedData(processedFormData);
	}

	protected FormData removeProcessedData(FormData formData) {
		return FormDataUtils.from(formData)
				.remove(ASSISTANT)
				.remove(ANLIEGEN_ID)
				.remove(KOMMUNALVERWALTUNG_ID)
				.remove(FormSolutionsAntragstellerMapper.POSTKORBHANDLE)
				.build();
	}
}
