package de.ozgcloud.eingang.xdomea;

import java.io.IOException;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPathFactoryConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class XMLHelper {

	private static final DocumentBuilder DOCUMENT_BUILDER = createDocumentBuilder();
	private static final XPathFactory X_PATH_FACTORY = createXPathFactory();

	public static Document parseDocument(IncomingFile incomingFile) {
		try (var inputStream = incomingFile.getContentStream()) {
			return DOCUMENT_BUILDER.parse(inputStream);
		} catch (SAXException | IOException e) {
			throw new TechnicalException("Failed to parse xml document!", e);
		}
	}

	public static XPathExpression compileXPathExpression(String xPathString) {
		try {
			return X_PATH_FACTORY.newXPath().compile(xPathString);
		} catch (XPathExpressionException e) {
			throw new TechnicalException("Failed to compile xpath expression!", e);
		}
	}

	private static DocumentBuilder createDocumentBuilder() {
		var documentBuilderFactory = DocumentBuilderFactory.newInstance();
		try {
			documentBuilderFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
			return documentBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new TechnicalException("Failed to configure document builder", e);
		}
	}

	private static XPathFactory createXPathFactory() {
		var xPathFactory = XPathFactory.newInstance();
		try {
			xPathFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, Boolean.TRUE);
			return xPathFactory;
		} catch (XPathFactoryConfigurationException e) {
			throw new TechnicalException("Failed to configure xpath factory!", e);
		}
	}
}
